@@ -18,7 +18,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -32,6 +32,7 @@
 #include "utils.h"
 #include <dbus/dbus-string.h>
 #include <dbus/dbus-internals.h>
+#include <dbus/dbus-message.h>
 #include <dbus/dbus-marshal-recursive.h>
 #include <string.h>
 
@@ -791,6 +792,9 @@ send_ack_reply (DBusConnection *connection,
 {
   DBusMessage *reply;
 
+  if (dbus_message_get_no_reply (message))
+    return TRUE;
+
   reply = dbus_message_new_method_return (message);
   if (reply == NULL)
     {
